--[[This is a module that can track accesses to a table.  It sets up a proxy
    for the table and returns that proxy back to the calling statement.  The proxy
    has a metatable set up on it with both __index and __newindex metamethods
    Those metamethods intercept every call to the table though the proxy and performs
    the correct actions on the actual table.  This is because the proxy is an empty 
    table.  So reads on empty fields call __index and write on empty fields call __newindex.
    Since the entire field is empty these two metamethods are called for all reads and writes.
    They then access the real table after doing some type of work on the keys or values.]]
local index = {}
local mt = {
  __index = function( table, key)
    print("*access to element " .. tostring(key))
    return table[index][key]
  end,

  __newindex = function(table, key, value)
    print("*update of element " .. tostring(key) .. " to " .. tostring(value))
    table[index][key] = value
  end,

  __pairs = function(table)
    return function(table, key)
      return next(table[index], key)
    end, table
  end
}

function track(table)
  local proxy = {}
  proxy[index] = table
  setmetatable(proxy, mt)
  return proxy
end










