--[[Take a table and make a proxy for that table that will be empty and
    therefore call the __index and __newindex metamethods for every access.
    The _newindex metamethod will return an error so that no new indices can
    be made in the table.  The _index method will look up the value in the
    table which has been assign to the __index metamethod]]
function readOnly (t)
  local proxy = {}
  local mt = {
    __index = t,
    __newindex = function(table, key, value)
      print("attempt to update a read-only table, operation " .. key .. " = " .. value .. " not performed")
    end
  }
  setmetatable(proxy, mt)
  return proxy
end







