--[[This is an example of the variables and function needed to make up
    a set and the code that uses those variables and functions.
    In this case the set functions and variables have not been
    moved to a separate module]]
local mt = {}
Set = {}

function Set.new (l)   -- 2nd version
  local set = {}
  setmetatable(set, mt)
  for _, v in ipairs(l) do set[v] = true end
  return set
end

function Set.union (a,b)
  local res = Set.new{}
  for k in pairs(a) do res[k] = true end
  for k in pairs(b) do res[k] = true end
  return res
end

function Set.intersection (a,b)
  local res = Set.new{}
  for k in pairs(a) do
    res[k] = b[k]
  end
  return res
end
function Set.tostring (set)
  local l = {}
  for e in pairs(set) do
    l[#l + 1] = e
  end
  return "{" .. table.concat(l, ", ") .. "}"
end

function Set.print (s)
  print(Set.tostring(s))
end

local s1 = Set.new{10, 20, 30, 50}
local s2 = Set.new{30, 1}

print(getmetatable(s1))
print(getmetatable(s2))

mt.__add = Set.union
mt.__mul = Set.intersection
--mt.__tostring = Set.tostring

s3 = s1 + s2

Set.print(s3)

Set.print((s1 + s2)*s1)

print((s1 + s2)*s1)







